@@ -84,7 +84,8 @@ void exit_with_usage(char *myname, char *message, char *arg)
 #endif
   printf("-ls [match]   or\n");
   printf("-list         Do nothing, just list our SockDir [on possible matches].\n");
-  printf("-L [file]     Turn on output logging. Set logfile name.\n");
+  printf("-L            Turn on output logging.\n");
+  printf("-Logfile file Set logfile name.\n");
   printf("-m            ignore $STY variable, do create a new screen session.\n");
   printf("-O            Choose optimal output rather than exact vt100 emulation.\n");
   printf("-p window     Preselect the named window if it exists.\n");
@@ -467,9 +468,10 @@ static const char cpmsg[] = "\
 \n\
 GNU Screen version %v\n\
 \n\
-Copyright (c) 2010 Juergen Weigert, Sadrul Habib Chowdhury\n\
-Copyright (c) 2008, 2009 Juergen Weigert, Michael Schroeder, Micah Cowan, Sadrul Habib Chowdhury\n\
-Copyright (c) 1993-2002, 2003, 2005, 2006, 2007 Juergen Weigert, Michael Schroeder\n\
+Copyright (c) 2015-2017 Juergen Weigert, Alexander Naumov, Amadeusz Slawinski\n\
+Copyright (c) 2010-2014 Juergen Weigert, Sadrul Habib Chowdhury\n\
+Copyright (c) 2008-2009 Juergen Weigert, Michael Schroeder, Micah Cowan, Sadrul Habib Chowdhury\n\
+Copyright (c) 1993-2007 Juergen Weigert, Michael Schroeder\n\
 Copyright (c) 1987 Oliver Laumann\n\
 \n\
 This program is free software; you can redistribute it and/or \
