@@ -1,11 +1,16 @@
-/* Copyright (c) 1993-2002
+/* Copyright (c) 2008, 2009
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
+ *      Micah Cowan (micah@cowan.name)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 1993-2002, 2003, 2005, 2006, 2007
  *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
  *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
  * Copyright (c) 1987 Oliver Laumann
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,16 +19,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * along with this program (see the file COPYING); if not, see
+ * http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  ****************************************************************
  */
 
-#include "rcs.h"
-RCS_ID("$Id: search.c,v 1.2 1994/05/31 12:32:57 mlschroe Exp $ FAU")
-
 #include <sys/types.h>
 
 #include "config.h"
@@ -65,7 +67,7 @@ int dir;
     }
   else
     Input((dir > 0 ? "/" : "?"), sizeof(markdata->isstr)-1, INP_COOKED,
-          (dir > 0 ? searchend : backsearchend), NULL);
+          (dir > 0 ? searchend : backsearchend), NULL, 0);
 }
 
 static void
@@ -130,23 +132,29 @@ char *data;	/* dummy */
     revto(x, y);
 }
 
+
+/*
+ * Search for a string that matches pattern.  The first character of the
+ * match must be on line y, between columns sx and ex (inclusive), but
+ * the rest of the match can extend beyond column ex and even onto the
+ * following line.  Returns the starting column of the first match found,
+ * or -1 if there's no match.
+ */
 static int
 matchword(pattern, y, sx, ex)
 char *pattern;
 int y, sx, ex;
 {
-  unsigned char *ip, *ipe, *cp, *pp;
-  struct mline *ml;
+  unsigned char *ip, *cp, *pp, *cpe;
+  int cy;
 
-  /* *sigh* to make WIN work */
   fore = ((struct markdata *)flayer->l_data)->md_window;
 
-  ml = WIN(y);
-  ip = ml->image + sx;
-  ipe = ml->image + flayer->l_width;
   for (;sx <= ex; sx++)
     {
-      cp = ip++;
+			cy = y;
+			cp = WIN(cy)->image + sx;
+			cpe = WIN(cy)->image + flayer->l_width;
       pp = (unsigned char *)pattern;
       for (;;)
 	{
@@ -157,8 +165,23 @@ int y, sx, ex;
 	  pp++;
 	  if (*pp == 0)
 	    return sx;
-	  if (cp == ipe)
-	    break;
+		if (cp == cpe) {
+		/*
+		* We have a partial match, but we've hit
+		* the end of this line.  Does it wrap onto
+		* the following line?  If not, we're done.
+		*/
+		if (*cp == ' ' || cy >= fore->w_histheight + flayer->l_height - 1)
+			break;
+
+		/*
+		* This line does wrap, so look on the next
+		* line for the rest of our match.
+		*/
+		cy++;
+		cp = WIN(cy)->image;
+		cpe = WIN(cy)->image + flayer->l_width;
+		}
 	}
     }
   return -1;
@@ -274,7 +297,7 @@ char *data;	/* dummy */
       break;
     case '\023':	/* CTRL-S */
     case '\022': 	/* CTRL-R */
-      if (markdata->isistrl >= sizeof(markdata->isistr))
+      if (markdata->isistrl >= (int)sizeof(markdata->isistr))
 	return;
       dir = (*p == '\023') ? 1 : -1;
       pos += dir;
@@ -288,8 +311,8 @@ char *data;	/* dummy */
       markdata->isistr[markdata->isistrl++] = *p;
       break;
     default:
-      if (*p < ' ' || markdata->isistrl >= sizeof(markdata->isistr)
-	  || markdata->isstrl >= sizeof(markdata->isstr) - 1)
+      if (*p < ' ' || markdata->isistrl >= (int)sizeof(markdata->isistr)
+	  || markdata->isstrl >= (int)sizeof(markdata->isstr) - 1)
 	return;
       markdata->isstr[markdata->isstrl++] = *p;
       markdata->isistr[markdata->isistrl++] = *p;
@@ -367,7 +390,7 @@ int dir;
   if (W2D(markdata->cy) == INPUTLINE)
     revto_line(markdata->cx, markdata->cy, INPUTLINE > 0 ? INPUTLINE - 1 : 1);
   Input(isprompts[dir + 1], sizeof(markdata->isstr) - 1, INP_RAW,
-        is_process, NULL);
+        is_process, NULL, 0);
   LGotoPos(flayer, markdata->cx, W2D(markdata->cy));
   flayer->l_x = markdata->cx;
   flayer->l_y = W2D(markdata->cy);
